<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:template="http://www.boddie.org.uk/ns/xmltools/template">

  <xsl:output indent="yes"/>



  <!-- Match the document itself. -->

  <xsl:template match="/">
    <!-- Process the elements. -->
    <xsl:apply-templates select="*"/>
  </xsl:template>



  <!-- Match element references. -->

  <xsl:template match="*[@template:element]">
    <xsl:call-template name="enter-element">
      <xsl:with-param name="other-elements" select="@template:element"/>
    </xsl:call-template>
  </xsl:template>

  <xsl:template name="enter-element">
    <xsl:param name="other-elements"/>
    <xsl:variable name="first-element" select="substring-before($other-elements, ',')"/>
    <xsl:variable name="remaining-elements" select="substring-after($other-elements, ',')"/>
    <xsl:choose>
      <xsl:when test="$first-element = ''">
        <xsl:call-template name="next-element">
          <xsl:with-param name="first-element" select="$other-elements"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="next-element">
          <xsl:with-param name="first-element" select="$first-element"/>
          <xsl:with-param name="remaining-elements" select="$remaining-elements"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="next-element">
    <xsl:param name="first-element"/>
    <xsl:param name="remaining-elements"/>
    <element>
      <xsl:attribute name="name"><xsl:value-of select="$first-element"/></xsl:attribute>
      <xsl:choose>
        <xsl:when test="$remaining-elements = ''">
          <xsl:call-template name="enter-attribute"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="enter-element">
            <xsl:with-param name="other-elements" select="$remaining-elements"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </element>
  </xsl:template>



  <!-- Match attributes. -->

  <xsl:template match="*[not(@template:element) and (@template:attribute or @template:attribute-field or @template:attribute-area or @template:attribute-button or @template:selector-field or @template:multiple-choice-field or @template:multiple-choice-list-field or @template:multiple-choice-value or @template:multiple-choice-list-value)]">
    <xsl:call-template name="enter-attribute"/>
  </xsl:template>

  <xsl:template name="enter-attribute">
    <xsl:choose>
      <xsl:when test="@template:attribute">
        <attribute>
          <xsl:attribute name="name"><xsl:value-of select="@template:attribute"/></xsl:attribute>
        </attribute>
      </xsl:when>
      <xsl:when test="@template:attribute-field">
        <attribute>
          <xsl:attribute name="name"><xsl:value-of select="@template:attribute-field"/></xsl:attribute>
        </attribute>
      </xsl:when>
      <xsl:when test="@template:attribute-area">
        <attribute>
          <xsl:attribute name="name"><xsl:value-of select="@template:attribute-area"/></xsl:attribute>
        </attribute>
      </xsl:when>
      <xsl:when test="@template:attribute-button">
        <attribute>
          <xsl:variable name="field-attr" select="substring-before(@template:attribute-button, ',')"/>
          <xsl:attribute name="name"><xsl:value-of select="$field-attr"/></xsl:attribute>
        </attribute>
      </xsl:when>
      <xsl:when test="@template:selector-field">
        <xsl:variable name="field-name" select="substring-before(@template:selector-field, ',')"/>
        <xsl:variable name="affected-element" select="substring-after(@template:selector-field, ',')"/>
        <selector>
          <xsl:choose>
            <xsl:when test="$field-name != ''">
              <xsl:attribute name="name"><xsl:value-of select="$field-name"/></xsl:attribute>
              <xsl:attribute name="element"><xsl:value-of select="$affected-element"/></xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="name"><xsl:value-of select="@template:selector-field"/></xsl:attribute>
            </xsl:otherwise>
          </xsl:choose>
        </selector>
      </xsl:when>
      <xsl:when test="@template:multiple-choice-field">
        <xsl:variable name="field-element" select="substring-before(@template:multiple-choice-field, ',')"/>
        <xsl:variable name="field-attr" select="substring-after(@template:multiple-choice-field, ',')"/>
        <xsl:choose>
          <xsl:when test="$field-element != '-'">
            <element type="multiple-choice">
              <xsl:attribute name="name"><xsl:value-of select="$field-element"/></xsl:attribute>
              <attribute>
                <xsl:attribute name="name"><xsl:value-of select="$field-attr"/></xsl:attribute>
              </attribute>
              <xsl:apply-templates select="*"/>
            </element>
          </xsl:when>
          <xsl:otherwise>
            <xsl:attribute name="type">multiple-choice</xsl:attribute>
            <attribute>
              <xsl:attribute name="name"><xsl:value-of select="$field-attr"/></xsl:attribute>
            </attribute>
            <xsl:apply-templates select="*"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="@template:multiple-choice-list-field">
        <xsl:variable name="field-element" select="substring-before(@template:multiple-choice-list-field, ',')"/>
        <xsl:variable name="field-list-info" select="substring-after(@template:multiple-choice-list-field, ',')"/>
        <xsl:variable name="field-list-element" select="substring-before($field-list-info, ',')"/>
        <xsl:variable name="field-list-attr" select="substring-after($field-list-info, ',')"/>
        <xsl:choose>
          <xsl:when test="$field-element != '-'">
            <element type="multiple-choice-list">
              <xsl:attribute name="name"><xsl:value-of select="$field-element"/></xsl:attribute>
              <attribute>
                <xsl:attribute name="name"><xsl:value-of select="$field-list-attr"/></xsl:attribute>
              </attribute>
              <xsl:apply-templates select="*"/>
            </element>
          </xsl:when>
          <xsl:otherwise>
            <xsl:attribute name="type">multiple-choice-list</xsl:attribute>
            <attribute>
              <xsl:attribute name="name"><xsl:value-of select="$field-list-attr"/></xsl:attribute>
            </attribute>
            <xsl:apply-templates select="*"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>
      <xsl:when test="@template:multiple-choice-value">
        <element type="multiple-choice-value">
          <xsl:variable name="field-element" select="substring-before(@template:multiple-choice-value, ',')"/>
          <xsl:variable name="field-attr-info" select="substring-after(@template:multiple-choice-value, ',')"/>
          <xsl:variable name="field-attr" select="substring-before($field-attr-info, ',')"/>
          <xsl:variable name="field-set-attr" select="substring-after($field-attr-info, ',')"/>
          <xsl:attribute name="name"><xsl:value-of select="$field-element"/></xsl:attribute>
          <xsl:attribute name="expr">@<xsl:value-of select="$field-attr"/> = ../@<xsl:value-of select="$field-attr"/></xsl:attribute>
          <xsl:attribute name="expr-attr"><xsl:value-of select="$field-set-attr"/></xsl:attribute>
          <attribute>
            <xsl:attribute name="name"><xsl:value-of select="$field-attr"/></xsl:attribute>
          </attribute>
        </element>
      </xsl:when>
      <xsl:when test="@template:multiple-choice-list-value">
        <element type="multiple-choice-list-value">
          <xsl:variable name="field-element" select="substring-before(@template:multiple-choice-list-value, ',')"/>
          <xsl:variable name="field-attr-info" select="substring-after(@template:multiple-choice-list-value, ',')"/>
          <xsl:variable name="field-attr" select="substring-before($field-attr-info, ',')"/>
          <xsl:variable name="field-set-attr" select="substring-after($field-attr-info, ',')"/>
          <xsl:attribute name="name"><xsl:value-of select="$field-element"/></xsl:attribute>
          <xsl:attribute name="expr">@value-is-set</xsl:attribute>
          <!-- Special attribute corresponding with the expression. -->
          <xsl:attribute name="expr-name">value-is-set</xsl:attribute>
          <xsl:attribute name="expr-attr"><xsl:value-of select="$field-set-attr"/></xsl:attribute>
          <attribute>
            <xsl:attribute name="name"><xsl:value-of select="$field-attr"/></xsl:attribute>
          </attribute>
        </element>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="*"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>



  <!-- Traverse unknown elements. -->

  <xsl:template match="*">
    <xsl:apply-templates select="*"/>
  </xsl:template>

</xsl:stylesheet>
